% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/microbiome_differential_abundance.R
\name{perform_and_visualize_DA}
\alias{perform_and_visualize_DA}
\title{Perform and Visualize Differential Abundance Analysis with edgeR or DESeq2}
\usage{
perform_and_visualize_DA(
  ps,
  method,
  group_var,
  contrast,
  pseudocount = 1,
  significance_level = 0.05,
  output_csv_path = "DA_results.csv",
  point_size = 3,
  target_glom = "Genus",
  palette = color_palette$extended_palette
)
}
\arguments{
\item{ps}{A `phyloseq` object containing microbiome data, including an OTU table and sample metadata.}

\item{method}{A character string specifying the method to use: \code{"edgeR"} or \code{"DESeq2"}.}

\item{group_var}{A string specifying the grouping variable in the sample metadata of the `phyloseq` object.}

\item{contrast}{A character vector specifying the levels to compare in the grouping variable (e.g., \code{c("Control", "Treated")}).}

\item{pseudocount}{A numeric value for pseudocount addition to handle zeros in count data. Default is \code{1}.}

\item{significance_level}{A numeric value specifying the significance threshold for adjusted p-values (FDR). Default is \code{0.05}.}

\item{output_csv_path}{A string specifying the path to save the results as a CSV file. Default is \code{"DA_results.csv"}.}

\item{point_size}{A numeric value specifying the size of points in the volcano plot. Default is \code{3}.}

\item{target_glom}{A string specifying the taxonomic rank to aggregate taxa for analysis. Default is \code{"Genus"}.}

\item{palette}{A character vector of color hex codes for plotting. Default is \code{color_palette$extended_palette}.}
}
\value{
A list with three components:
\itemize{
  \item \code{results}: A data frame containing differential abundance analysis results, including \code{logFC}, \code{pvalue}, \code{FDR}, \code{diff_abn}, and \code{treatment}.
  \item \code{ps_significant}: A `phyloseq` object containing only the significant OTUs/ASVs.
  \item \code{plot}: A ggplot2 object representing the volcano plot of the differential abundance results.
}
}
\description{
This function performs differential abundance analysis using either edgeR or DESeq2, preprocesses the data,
and generates a volcano plot based on microbiome data.
}
\details{
This function preprocesses microbiome count data, performs differential abundance analysis using 
edgeR or DESeq2, and visualizes the results as a volcano plot. Significant OTUs/ASVs are filtered 
based on the specified FDR threshold and included in the returned phyloseq object.
}
\examples{
\dontrun{
# Example with edgeR
results_edgeR <- perform_and_visualize_DA(
  ps = ps,
  method = "edgeR",
  group_var = "Treatment",
  contrast = c("Control", "Treated"),
  output_csv_path = "DA_edgeR.csv",
  target_glom = "Genus",
  significance_level = 0.05
)
print(results_edgeR$plot)

# Example with DESeq2
results_DESeq2 <- perform_and_visualize_DA(
  ps = ps,
  method = "DESeq2",
  group_var = "Treatment",
  contrast = c("Control", "Treated"),
  output_csv_path = "DA_DESeq2.csv",
  target_glom = "Genus",
  significance_level = 0.05
)
print(results_DESeq2$plot)

# Access significant taxa
significant_ps <- results_edgeR$ps_significant
head(results_edgeR$results)  # View significant taxa
}

}
